@@ -75,14 +75,12 @@ impl System {
 
         let hidpi_factor = platform.hidpi_factor();
         let font_size = (13.0 * hidpi_factor) as f32;
-        imgui.fonts().add_font(&[
-            FontSource::DefaultFontData {
-                config: Some(FontConfig {
-                    size_pixels: font_size,
-                    ..FontConfig::default()
-                }),
-            },
-        ]);
+        imgui.fonts().add_font(&[FontSource::DefaultFontData {
+            config: Some(FontConfig {
+                size_pixels: font_size,
+                ..FontConfig::default()
+            }),
+        }]);
         imgui.io_mut().font_global_scale = (1.0 / hidpi_factor) as f32;
 
         let renderer = Renderer::init(&mut imgui, &display).expect("Failed to initialize renderer");
@@ -104,6 +102,12 @@ impl System {
     }
 }
 
+impl Default for System {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 //https://stackoverflow.com/questions/56701736/how-to-correctly-translate-mouse-coords-to-opengl-coords
 fn _cord_to_gl(x: f32, y: f32, width: f32, height: f32) -> (f32, f32) {
     let x = 2.0 * (x / width) - 1.0;
@@ -157,7 +161,7 @@ impl System {
                         .render(&mut target, draw_data)
                         .expect("Rendering failed");
                     target.finish().expect("Failed to swap buffers");
-                }
+                },
                 Event::WindowEvent {
                     event: WindowEvent::CloseRequested,
                     ..
